@@ -86,7 +86,7 @@ module Test
       end
 
       ##
-      # Passes if +expected+ == +actual.
+      # Passes if +expected+ == +actual+.
       #
       # Note that the ordering of arguments is important, since a helpful
       # error message is generated when this one fails that tells you the
@@ -118,11 +118,7 @@ EOT
         begin
           assert_block(full_message) { expected == actual }
         rescue AssertionFailedError => failure
-          failure.expected = expected
-          failure.actual = actual
-          failure.inspected_expected = AssertionMessage.convert(expected)
-          failure.inspected_actual = AssertionMessage.convert(actual)
-          failure.user_message = message
+          _set_failed_information(failure, expected, actual, message)
           raise failure # For JRuby. :<
         end
       end
@@ -149,11 +145,19 @@ EOT
         assert_expected_exception = Proc.new do |*_args|
           message, assert_exception_helper, actual_exception = _args
           expected = assert_exception_helper.expected_exceptions
+          diff = AssertionMessage.delayed_diff(expected, actual_exception)
           full_message = build_message(message,
-                                       "<?> exception expected but was\n?",
-                                       expected, actual_exception)
-          assert_block(full_message) do
-            expected == [] or assert_exception_helper.expected?(actual_exception)
+                                       "<?> exception expected but was\n<?>.?",
+                                       expected, actual_exception, diff)
+          begin
+            assert_block(full_message) do
+              expected == [] or
+                assert_exception_helper.expected?(actual_exception)
+            end
+          rescue AssertionFailedError => failure
+            _set_failed_information(failure, expected, actual_exception,
+                                    message)
+            raise failure # For JRuby. :<
           end
         end
         _assert_raise(assert_expected_exception, *args, &block)
@@ -183,7 +187,7 @@ EOT
           expected = assert_exception_helper.expected_exceptions
           full_message = build_message(message,
                                        "<?> family exception expected " +
-                                       "but was\n?",
+                                       "but was\n<?>.",
                                        expected, actual_exception)
           assert_block(full_message) do
             assert_exception_helper.expected?(actual_exception, :kind_of?)
@@ -1423,6 +1427,15 @@ EOT
         end
       end
 
+      private
+      def _set_failed_information(failure, expected, actual, user_message)
+        failure.expected = expected
+        failure.actual = actual
+        failure.inspected_expected = AssertionMessage.convert(expected)
+        failure.inspected_actual = AssertionMessage.convert(actual)
+        failure.user_message = user_message
+      end
+
       class AssertionMessage
         @use_pp = true
         class << self
@@ -1515,30 +1528,22 @@ EOT
           end
 
           def convert(object)
-            case object
-            when Exception
-              <<EOM.chop
-Class: <#{convert(object.class)}>
-Message: <#{convert(object.message)}>
----Backtrace---
-#{Util::BacktraceFilter.filter_backtrace(object.backtrace).join("\n")}
----------------
-EOM
-            else
-              inspector = Inspector.new(object)
-              if use_pp
-                begin
-                  require 'pp' unless defined?(PP)
-                  begin
-                    return PP.pp(inspector, '').chomp
-                  rescue NameError
-                  end
-                rescue LoadError
-                  self.use_pp = false
-                end
-              end
-              inspector.inspect
+            if object.is_a?(Exception)
+              object = AssertExceptionHelper::WrappedException.new(object)
             end
+            inspector = Inspector.new(object)
+            if use_pp
+              begin
+                require 'pp' unless defined?(PP)
+                begin
+                  return PP.pp(inspector, '').chomp
+                rescue NameError
+                end
+              rescue LoadError
+                self.use_pp = false
+              end
+            end
+            inspector.inspect
           end
         end
 
@@ -1550,6 +1555,19 @@ EOM
               inspected_objects[object.object_id] ||=
                 new(object, inspected_objects)
             end
+
+            @@inspector_classes = []
+            def inspector_classes
+              @@inspector_classes
+            end
+
+            def register_inspector_class(inspector_class)
+              @@inspector_classes << inspector_class
+            end
+
+            def unregister_inspector_class(inspector_class)
+              @@inspector_classes.delete(inspector_class)
+            end
           end
 
           attr_reader :object
@@ -1583,17 +1601,18 @@ EOM
 
           private
           def inspect_target
-            if HashInspector.target?(@object)
-              HashInspector.new(@object, @inspected_objects)
-            elsif ArrayInspector.target?(@object)
-              ArrayInspector.new(@object, @inspected_objects)
-            else
-              @object
+            self.class.inspector_classes.each do |inspector_class|
+              if inspector_class.target?(@object)
+                return inspector_class.new(@object, @inspected_objects)
+              end
             end
+            @object
           end
         end
 
         class HashInspector
+          Inspector.register_inspector_class(self)
+
           class << self
             def target?(object)
               object.is_a?(Hash) or object == ENV
@@ -1646,6 +1665,8 @@ EOM
         end
 
         class ArrayInspector
+          Inspector.register_inspector_class(self)
+
           class << self
             def target?(object)
               object.is_a?(Array)
@@ -1734,7 +1755,30 @@ EOM
           def result(parameters)
             raise "The number of parameters does not match the number of substitutions." if(parameters.size != count)
             params = parameters.dup
-            @parts.collect{|e| e == '?' ? params.shift : e.gsub(/\\\?/m, '?')}.join('')
+            expanded_template = ""
+            @parts.each do |part|
+              if part == '?'
+                encoding_safe_concat(expanded_template, params.shift)
+              else
+                expanded_template << part.gsub(/\\\?/m, '?')
+              end
+            end
+            expanded_template
+          end
+
+          private
+          if Object.const_defined?(:Encoding)
+            def encoding_safe_concat(buffer, parameter)
+              if Encoding.compatible?(buffer.encoding, parameter.encoding)
+                buffer << parameter
+              else
+                buffer << parameter.dup.force_encoding(buffer.encoding)
+              end
+            end
+          else
+            def encoding_safe_concat(buffer, parameter)
+              buffer << parameter
+            end
           end
         end
 
@@ -1761,7 +1805,7 @@ EOM
         def to_s
           message_parts = []
           if (@head)
-            head = @head.to_s 
+            head = @head.to_s
             unless(head.empty?)
               message_parts << add_period(head)
             end
@@ -1774,13 +1818,14 @@ EOM
 
       class AssertExceptionHelper
         class WrappedException
+          attr_reader :exception
           def initialize(exception)
             @exception = exception
           end
 
           def inspect
             if default_inspect?
-              "#{@exception.class.inspect}(#{@exception.message.inspect})"
+              "#{@exception.class.inspect}(<#{@exception.message}>)"
             else
               @exception.inspect
             end
