@@ -8,6 +8,7 @@
 
 require 'test/unit/color-scheme'
 require 'test/unit/code-snippet-fetcher'
+require 'test/unit/diff'
 require 'test/unit/ui/testrunner'
 require 'test/unit/ui/testrunnermediator'
 require 'test/unit/ui/console/outputlevel'
@@ -168,8 +169,7 @@ module Test
           end
 
           def output_fault_in_detail(fault)
-            if @use_color and fault.is_a?(Failure) and
-                fault.inspected_expected and fault.inspected_actual
+            if fault.is_a?(Failure)
               output_single(fault.label, fault_color(fault))
               output(":")
               output(fault.test_name)
@@ -429,7 +429,7 @@ module Test
           def guess_color_availability
             return false unless @output.tty?
             case ENV["TERM"]
-            when /term(?:-color)?\z/, "screen"
+            when /term(?:-(?:256)?color)?\z/, "screen"
               true
             else
               return true if ENV["EMACS"] == "t"
